/**
   This program writes pixel data to a PPM file using the FrameBuffer class.
   The pixel data is a sequence of red, then green, then blue pixels.

   In addition, this example creates two "viewports" within the framebuffer.
   One viewport holds random pixels and the other viewport contains the Yoda picture.
*/
import java.util.Random;
import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;

public class Example_3
{
   public static void main(String[] args)
   {
      String filename = "Example_3.ppm";

      int height = 600;
      int width  = 601;
      FrameBuffer fb = new FrameBuffer(height, width);

      for (int i = 0; i < height*width*3; i+=9)
      {
         fb.pixel_buffer[i+0] = 255;  // a red pixel
         fb.pixel_buffer[i+1] = 0;
         fb.pixel_buffer[i+2] = 0;

         fb.pixel_buffer[i+3] = 0;    // followed by a green pixel
         fb.pixel_buffer[i+4] = 255;
         fb.pixel_buffer[i+5] = 0;

         fb.pixel_buffer[i+6] = 0;    // followed by a green pixel
         fb.pixel_buffer[i+7] = 0;
         fb.pixel_buffer[i+8] = 255;
      }


      // Create a "viewport" within the framebuffer to hold random pixels.
      fb.setViewport(64, 64, 256, 128); // upper-left-hand-corner, width, height

      Random random = new Random();
      for (int y = 0; y < 128; y++)
      {
         for (int x = 0; x < 256; x++)
         {
            int r = random.nextInt(256);
            int g = random.nextInt(256);
            int b = random.nextInt(256);
            fb.setPixelVP(x, y, new Color(r, g, b));
         }
      }


      // Create a "viewport" within the framebuffer to hold the Yoda image.
      fb.setViewport(300, 300, 256, 256); // upper-left-hand-corner, width, height

      // Read the pixel data in a RAW file.
      String inputFileName = "Yoda.raw";
      try
      {
         FileInputStream fis = new FileInputStream(inputFileName);

         // Create a data array.
         byte[] pixelData = new byte[3];
         // Read pixel data, one pixel at a time, from the RAW file.
         for (int y = 0; y < 256; y++)
         {
            for (int x = 0; x < 256; x++)
            {
               if ( fis.read(pixelData, 0, 3) != 3 )
               {
                  System.err.printf("ERROR! Could not load %s\n", inputFileName);
                  System.exit(-1);
               }
               int r = pixelData[0];
               int g = pixelData[1];
               int b = pixelData[2];
               if (r < 0) r = 256+r;  // convert from signed byte to unsigned byte
               if (g < 0) g = 256+g;
               if (b < 0) b = 256+b;
               fb.setPixelVP(x, y, new Color(r, g, b));
            }
         }

         fis.close();
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not read %s\n", inputFileName);
         System.exit(-1);
      }

      fb.dumpFB2File( filename );
   }
}